/* tslint:disable */
/* eslint-disable */
/**
* @returns {any}
*/
declare function get_version(): any;

declare function experimental_build(
	entry_path: string, fs_resolve_js: (path: string) => string | undefined, minify: boolean
): {Ok: BuildOutput} | {Err: FailedBuildOutput}



declare function check(entry_path: string, fs_resolver_js: (path: string) => string | undefined): WASMCheckOutput



declare function check_with_options(entry_path: string, fs_resolver_js: (path: string) => string | undefined, options: TypeCheckOptions): WASMCheckOutput



declare function run_cli(
	cli_arguments: string[],
	read_from_file: (path: string) => string | undefined,
	write_to_file: (path: string, content: string) => void,
	cli_input_resolver: (prompt: string) => string | undefined
): void



declare function parse_expression(input: string): Expression | [string, Span]



declare function parse_module(input: string): Module | [string, Span]



declare function parse_module_and_into_string(
	input: string,
	parse_options: ParseOptions,
	to_string_options: ToStringOptions
): string | [string, Span]



declare function just_imports(input: string): string | [string, Span]


interface FailedBuildOutput {
    diagnostics: DiagnosticsContainer;
}

interface BuildOutput {
    outputs: Output[];
    diagnostics: DiagnosticsContainer;
}

interface Output {
    output_path: string;
    content: string;
    mappings: string;
}

type DiagnosticsContainer = Diagnostic[];

type Diagnostic = { reason: string; kind: DiagnosticKind } | { reason: string; position: SpanWithSource; kind: DiagnosticKind } | { reason: string; position: SpanWithSource; labels: [string, SpanWithSource | null][]; kind: DiagnosticKind };

type DiagnosticKind = "error" | "warning" | "info";

interface TypeCheckOptions {
    constant_parameters?: boolean;
    allow_elided_arguments?: boolean;
    allow_extra_arguments?: boolean;
    constant_function_declarations?: boolean;
    strict_casts?: boolean;
    debug_types?: boolean;
    allow_cast?: boolean;
    store_expression_type_mappings?: boolean;
    parse_comments?: boolean;
    lsp_mode?: boolean;
}

type Expression = { NumberLiteral: [NumberRepresentation, Span] } | { StringLiteral: [string, Quoted, Span] } | { BooleanLiteral: [boolean, Span] } | { RegexLiteral: { pattern: string; flags: string | null; position: Span } } | { ArrayLiteral: [ArrayElement[], Span] } | { ObjectLiteral: ObjectLiteral } | { TemplateLiteral: TemplateLiteral } | { ParenthesizedExpression: [MultipleExpression, Span] } | { BinaryOperation: { lhs: Expression; operator: BinaryOperator; rhs: Expression; position: Span } } | { SpecialOperators: [SpecialOperators, Span] } | { UnaryOperation: { operator: UnaryOperator; operand: Expression; position: Span } } | { Assignment: { lhs: LHSOfAssignment; rhs: Expression; position: Span } } | { BinaryAssignmentOperation: { lhs: VariableOrPropertyAccess; operator: BinaryAssignmentOperator; rhs: Expression; position: Span } } | { UnaryPrefixAssignmentOperation: { operator: UnaryPrefixAssignmentOperator; operand: VariableOrPropertyAccess; position: Span } } | { UnaryPostfixAssignmentOperation: { operand: VariableOrPropertyAccess; operator: UnaryPostfixAssignmentOperator; position: Span } } | { VariableReference: [string, Span] } | { ThisReference: Span } | { SuperExpression: [SuperReference, Span] } | { NewTarget: Span } | { DynamicImport: { path: Expression; options: Expression | null; position: Span } } | { PropertyAccess: { parent: Expression; property: PropertyReference; is_optional: boolean; position: Span } } | { Index: { indexee: Expression; indexer: MultipleExpression; is_optional: boolean; position: Span } } | { FunctionCall: { function: Expression; type_arguments: TypeAnnotation[] | null; arguments: FunctionArgument[]; is_optional: boolean; position: Span } } | { ConstructorCall: { constructor: Expression; type_arguments: TypeAnnotation[] | null; arguments: FunctionArgument[] | null; position: Span } } | { ConditionalTernary: { condition: Expression; truthy_result: Expression; falsy_result: Expression; position: Span } } | { ArrowFunction: ArrowFunction } | { ExpressionFunction: ExpressionFunction } | { ClassExpression: ClassDeclaration<ExpressionPosition> } | { Null: Span } | { Comment: { content: string; on: Expression; position: Span; is_multiline: boolean; prefix: boolean } } | { JSXRoot: JSXRoot } | { IsExpression: IsExpression } | { Marker: { marker_id: Marker<Expression>; position: Span } };

type PropertyReference = { Standard: { property: string; is_private: boolean } } | { Marker: Marker<PropertyReference> };

type MultipleExpression = { Multiple: { lhs: MultipleExpression; rhs: Expression; position: Span } } | { Single: Expression };

type SpecialOperators = { Satisfies: { value: Expression; type_annotation: TypeAnnotation } } | { In: { lhs: InExpressionLHS; rhs: Expression } } | { InstanceOf: { lhs: Expression; rhs: Expression } } | { Is: { value: Expression; type_annotation: TypeAnnotation } } | { NonNullAssertion: Expression } | { AsCast: { value: Expression; rhs: TypeOrConst } };

type TypeOrConst = { Type: TypeAnnotation } | { Const: Span };

type InExpressionLHS = { PrivateProperty: string } | { Expression: Expression };

type FunctionArgument = { Spread: [Expression, Span] } | { Standard: Expression } | { Comment: { content: string; is_multiline: boolean; position: Span } };

type ArrayElement = FunctionArgument | null;

type SuperReference = { Call: { arguments: FunctionArgument[] } } | { PropertyAccess: { property: string } } | { Index: { indexer: Expression } };

type VariableKeyword = "Const" | "Let" | "Var";

type ExpressionPosition = VariableIdentifier | null;

interface StatementPosition {
    identifier: VariableIdentifier;
    declare: boolean;
}

type NumberRepresentation = "Infinity" | "NegativeInfinity" | "NaN" | { Hex: { sign: NumberSign; value: number } } | { Bin: { sign: NumberSign; value: number } } | { Octal: { sign: NumberSign; value: number } } | { Number: number } | { Exponential: { sign: NumberSign; value: number; exponent: number } } | { BigInt: [NumberSign, string] };

type NumberSign = "Positive" | "Negative";

type Comments = "All" | "JustDocumentation" | "None";

interface ToStringOptions {
    pretty?: boolean;
    trailing_semicolon?: boolean;
    single_statement_on_new_line?: boolean;
    include_type_annotations?: boolean;
    include_decorators?: boolean;
    comments?: Comments;
    indent_with?: string;
    expect_jsx?: boolean;
    expect_markers?: boolean;
    max_line_length?: number;
}

interface ParseOptions {
    jsx?: boolean;
    type_annotations?: boolean;
    type_definition_module?: boolean;
    special_jsx_attributes?: boolean;
    decorators?: boolean;
    comments?: Comments;
    is_expressions?: boolean;
    custom_function_headers?: boolean;
    buffer_size?: number;
    stack_size?: number | null;
    record_keyword_positions?: boolean;
    interpolation_points?: boolean;
    partial_syntax?: boolean;
}

type Quoted = "Single" | "Double";


	interface ArrowFunction extends FunctionBase {
		header: IsAsync,
		body: ExpressionOrBlock
	}


type IsAsync = boolean;

type ExpressionOrBlock = { Expression: Expression } | { Block: Block };

type VariableOrPropertyAccess = { Variable: [string, Span] } | { PropertyAccess: { parent: Expression; property: PropertyReference; position: Span } } | { Index: { indexee: Expression; indexer: MultipleExpression; position: Span } };

type LHSOfAssignment = { VariableOrPropertyAccess: VariableOrPropertyAccess } | { ArrayDestructuring: [WithComment<ArrayDestructuringField>[], Span] } | { ObjectDestructuring: [WithComment<ObjectDestructuringField>[], Span] };

interface ForLoopStatement {
    condition: ForLoopCondition;
    inner: BlockOrSingleStatement;
    position: Span;
}

type ForLoopStatementInitializer = { VariableDeclaration: VariableDeclaration } | { VarStatement: VarVariableStatement } | { Expression: MultipleExpression };

type ForLoopCondition = { ForOf: { keyword: VariableKeyword | null; variable: WithComment<VariableField>; of: Expression; is_await: boolean; position: Span } } | { ForIn: { keyword: VariableKeyword | null; variable: WithComment<VariableField>; in: MultipleExpression; position: Span } } | { Statements: { initialiser: ForLoopStatementInitializer | null; condition: MultipleExpression | null; afterthought: MultipleExpression | null; position: Span } };

interface TypeAlias {
    name: StatementPosition;
    parameters: TypeParameter[] | null;
    references: TypeAnnotation;
    position: Span;
}

interface TypeParameter {
    name: string;
    default: TypeAnnotation | null;
    extends: TypeAnnotation | null;
    position: Span;
    is_constant: boolean;
}

interface VariableDeclarationItem<TExpr> {
    name: WithComment<VariableField>;
    type_annotation: TypeAnnotation | null;
    expression: TExpr;
    position: Span;
}

type VariableDeclaration = { ConstDeclaration: { declarations: VariableDeclarationItem<Expression>[]; position: Span } } | { LetDeclaration: { declarations: VariableDeclarationItem<Expression | null>[]; position: Span } };

type VariableDeclarationKeyword = "Const" | "Let";

type TypeAnnotation = { Name: [string, Span] } | { CommonName: [CommonTypes, Span] } | { NamespacedName: [string, string, Span] } | { NameWithGenericArguments: [string, TypeAnnotation[], Span] } | { Union: [TypeAnnotation[], Span] } | { Intersection: [TypeAnnotation[], Span] } | { StringLiteral: [string, Quoted, Span] } | { NumberLiteral: [NumberRepresentation, Span] } | { BooleanLiteral: [boolean, Span] } | { ArrayLiteral: [TypeAnnotation, Span] } | { FunctionLiteral: { type_parameters: TypeParameter[] | null; parameters: TypeAnnotationFunctionParameters; return_type: TypeAnnotation; position: Span } } | { ConstructorLiteral: { type_parameters: TypeParameter[] | null; parameters: TypeAnnotationFunctionParameters; return_type: TypeAnnotation; position: Span } } | { ObjectLiteral: [WithComment<Decorated<InterfaceMember>>[], Span] } | { TupleLiteral: [[TupleElementKind, AnnotationWithBinder][], Span] } | { TemplateLiteral: [TemplateLiteralPart<AnnotationWithBinder>[], Span] } | { Readonly: [TypeAnnotation, Span] } | { Index: [TypeAnnotation, TypeAnnotation, Span] } | { KeyOf: [TypeAnnotation, Span] } | { ParenthesizedReference: [TypeAnnotation, Span] } | { Conditional: { condition: TypeCondition; resolve_true: TypeConditionResult; resolve_false: TypeConditionResult; position: Span } } | { Symbol: { unique: boolean; name: string | null; position: Span } } | { Decorated: [Decorator, TypeAnnotation, Span] } | { Marker: [Marker<TypeAnnotation>, Span] };

type AnnotationWithBinder = { Annotated: { name: string; ty: TypeAnnotation; position: Span } } | { NoAnnotation: TypeAnnotation };

type TupleElementKind = "Standard" | "Spread" | "Optional";

type TypeCondition = { Extends: { ty: TypeAnnotation; extends: TypeAnnotation; position: Span } } | { Is: { ty: TypeAnnotation; is: TypeAnnotation; position: Span } };

type CommonTypes = "String" | "Number" | "Boolean";

type TypeConditionResult = { Infer: [TypeAnnotation, Span] } | { Reference: TypeAnnotation };

interface TypeAnnotationFunctionParameters {
    parameters: TypeAnnotationFunctionParameter[];
    rest_parameter: TypeAnnotationSpreadFunctionParameter | null;
    position: Span;
}

interface TypeAnnotationFunctionParameter {
    decorators: Decorator[];
    name: WithComment<VariableField> | null;
    type_annotation: TypeAnnotation;
    is_optional: boolean;
    position: Span;
}

interface TypeAnnotationSpreadFunctionParameter {
    decorators: Decorator[];
    name: string;
    type_annotation: TypeAnnotation;
    position: Span;
}


	type Marker<T> = number;


interface Decorator {
    name: string[];
    arguments: Expression[] | null;
    position: Span;
}

interface Decorated<T> {
    decorators: Decorator[];
    on: T;
    position: Span;
}

type SpreadParameterName = VariableField;

type ImportedItems = { Parts: ImportPart[] | null } | { All: { under: VariableIdentifier } };

interface ImportDeclaration {
    is_deferred: boolean;
    is_type_annotation_import_only: boolean;
    default: VariableIdentifier | null;
    items: ImportedItems;
    from: ImportLocation;
    with: ObjectLiteral | null;
    position: Span;
    reversed: boolean;
}

type ImportExportName = { Reference: string } | { Quoted: [string, Quoted] } | { Marker: Marker<ImportExportName> };

type ImportPart = { Name: VariableIdentifier } | { NameWithAlias: { name: string; alias: ImportExportName; position: Span } } | { PrefixComment: [string, ImportPart | null, Span] } | { PostfixComment: [ImportPart, string, Span] };

interface IsExpression {
    matcher: MultipleExpression;
    branches: [TypeAnnotation, ExpressionOrBlock][];
    position: Span;
}

interface Parameter<V> {
    visibility: V;
    name: WithComment<VariableField>;
    type_annotation: TypeAnnotation | null;
    additionally: ParameterData | null;
    position: Span;
}

type ParameterData = "Optional" | { WithDefaultValue: Expression };

interface SpreadParameter {
    name: SpreadParameterName;
    type_annotation: TypeAnnotation | null;
    position: Span;
}

interface FunctionParameters<L, V> {
    leading: L;
    parameters: Parameter<V>[];
    rest_parameter: SpreadParameter | null;
    position: Span;
}

interface ThisParameter {
    constraint: TypeAnnotation;
    position: Span;
}

interface SuperParameter {
    constraint: TypeAnnotation;
    position: Span;
}

type AlwaysPublic = null;

type PublicOrPrivate = "Public" | "Private";

type PropertyKey<T> = { Ident: [string, Span, T] } | { StringLiteral: [string, Quoted, Span] } | { NumberLiteral: [NumberRepresentation, Span] } | { Computed: [Expression, Span] };


	interface ClassConstructor extends FunctionBase {
		body: FunctionBody,
		parameters: FunctionParameters<[ThisParameter | null, SuperParameter | null], Visibility>,
	}

	interface ClassFunction extends FunctionBase {
		header: MethodHeader,
		name: WithComment<PropertyKey<PublicOrPrivate>>
		parameters: FunctionParameters<ThisParameter | null, null>,
		body: FunctionBody,
	}



	interface ObjectLiteralMethod extends FunctionBase {
		header: MethodHeader,
		body: Block,
		name: WithComment<PropertyKey<AlwaysPublic>>,
		parameters: FunctionParameters<ThisParameter | null, null>
	}


type ExceptionVarField = WithComment<VariableField>;

type IsStatic = boolean;

type ClassMember = { Constructor: ClassConstructor } | { Method: [IsStatic, ClassFunction] } | { Property: [IsStatic, ClassProperty] } | { StaticBlock: Block } | { Indexer: { name: string; indexer_type: TypeAnnotation; return_type: TypeAnnotation; is_readonly: boolean; position: Span } } | { Comment: [string, boolean, Span] };

interface ClassProperty {
    is_readonly: boolean;
    key: WithComment<PropertyKey<PublicOrPrivate>>;
    type_annotation: TypeAnnotation | null;
    value: Expression | null;
    position: Span;
}

type ExportDeclaration = { Variable: { exported: Exportable; position: Span } } | { Default: { expression: Expression; position: Span } } | { DefaultFunction: { is_async: boolean; identifier: VariableIdentifier | null; parameters: TypeAnnotationFunctionParameters; return_type: TypeAnnotation | null; position: Span } };

type Exportable = { Class: ClassDeclaration<StatementPosition> } | { Function: StatementFunction } | { Variable: VariableDeclaration } | { Interface: InterfaceDeclaration } | { TypeAlias: TypeAlias } | { Parts: ExportPart[] } | { ImportAll: { as: VariableIdentifier | null; from: ImportLocation } } | { ImportParts: { parts: ExportPart[]; from: ImportLocation; type_definitions_only: boolean } };

type ExportPart = { Name: VariableIdentifier } | { NameWithAlias: { name: string; alias: ImportExportName; position: Span } } | { PrefixComment: [string, ExportPart | null, Span] } | { PostfixComment: [ExportPart, string, Span] };

interface ObjectLiteral {
    members: ObjectLiteralMember[];
    position: Span;
}

type ObjectLiteralMember = { Spread: [Expression, Span] } | { Shorthand: [string, Span] } | { Property: { key: WithComment<PropertyKey<AlwaysPublic>>; assignment: boolean; value: Expression; position: Span } } | { Method: ObjectLiteralMethod };

type JSXRoot = { Element: JSXElement } | { Fragment: JSXFragment };

interface JSXElement {
    tag_name: string;
    attributes: JSXAttribute[];
    children: JSXElementChildren;
    position: Span;
}

type JSXElementChildren = { Children: JSXNode[] } | "SelfClosing";

interface JSXFragment {
    children: JSXNode[];
    position: Span;
}

type JSXNode = { Element: JSXElement } | { TextNode: [string, Span] } | { InterpolatedExpression: [FunctionArgument, Span] } | { Comment: [string, Span] } | "LineBreak";

type JSXAttribute = { Static: [string, string, Span] } | { Dynamic: [string, Expression, Span] } | { BooleanAttribute: [string, Span] } | { Spread: [Expression, Span] } | { Shorthand: Expression };

interface TryCatchStatement {
    try_inner: Block;
    catch_inner: Block | null;
    exception_var: [ExceptionVarField, TypeAnnotation | null] | null;
    finally_inner: Block | null;
    position: Span;
}

interface DeclareVariableDeclaration {
    keyword: VariableKeyword;
    declarations: VariableDeclarationItem<Expression | null>[];
    position: Span;
    decorators: Decorator[];
}


	interface StatementFunction extends FunctionBase {
		header: FunctionHeader,
		parameters: FunctionParameters<ThisParameter | null, null>,
		body: Block,
		name: StatementPosition
	}


type StatementOrDeclaration = { Statement: Statement } | { Declaration: Declaration } | { Marker: [Marker<Statement>, Span] };

type Block = [StatementOrDeclaration[], Span];

type BlockOrSingleStatement = { Braced: Block } | { SingleStatement: Statement };

type Declaration = { Variable: VariableDeclaration } | { Function: Decorated<StatementFunction> } | { Class: Decorated<ClassDeclaration<StatementPosition>> } | { Enum: Decorated<EnumDeclaration> } | { Interface: Decorated<InterfaceDeclaration> } | { TypeAlias: TypeAlias } | { DeclareVariable: DeclareVariableDeclaration } | { Namespace: Namespace } | { Import: ImportDeclaration } | { Export: Decorated<ExportDeclaration> };

type ImportLocation = { Quoted: [string, Quoted] } | { Marker: Marker<ImportLocation> };

interface Module {
    items: StatementOrDeclaration[];
    span: Span;
}

interface SwitchStatement {
    case: MultipleExpression;
    branches: SwitchBranch[];
    position: Span;
}

type SwitchBranch = { Default: StatementOrDeclaration[] } | { Case: [Expression, StatementOrDeclaration[]] };

interface WhileStatement {
    condition: MultipleExpression;
    inner: BlockOrSingleStatement;
    position: Span;
}

interface DoWhileStatement {
    condition: MultipleExpression;
    inner: BlockOrSingleStatement;
    position: Span;
}

interface Namespace {
    name: string;
    inner: Block;
    position: Span;
}

type WithComment<T> = { None: T } | { PrefixComment: [string, T, Span] } | { PostfixComment: [T, string, Span] };

type VariableIdentifier = { Standard: [string, Span] } | { Marker: [VariableIdentifier, Span] };

type VariableField = { Name: VariableIdentifier } | { Array: [WithComment<ArrayDestructuringField>[], Span] } | { Object: [WithComment<ObjectDestructuringField>[], Span] };

type ArrayDestructuringField = { Spread: [VariableField, Span] } | { Name: [VariableField, Expression | null] } | { Comment: { content: string; is_multiline: boolean; position: Span } } | "None";

type ObjectDestructuringField = { Name: [VariableIdentifier, Expression | null, Span] } | { Spread: [VariableIdentifier, Span] } | { Map: { from: PropertyKey<AlwaysPublic>; name: WithComment<VariableField>; default_value: Expression | null; position: Span } };

interface ClassDeclaration<T> {
    name: T;
    type_parameters: TypeParameter[] | null;
    extends: Expression | null;
    implements: TypeAnnotation[] | null;
    members: Decorated<ClassMember>[];
    position: Span;
}

interface IfStatement {
    condition: MultipleExpression;
    inner: BlockOrSingleStatement;
    else_conditions: ConditionalElseStatement[];
    trailing_else: UnconditionalElseStatement | null;
    position: Span;
}

interface ConditionalElseStatement {
    condition: MultipleExpression;
    inner: BlockOrSingleStatement;
    position: Span;
}

interface UnconditionalElseStatement {
    inner: BlockOrSingleStatement;
    position: Span;
}

type Statement = { Expression: MultipleExpression } | { Block: Block } | { Debugger: Span } | { If: IfStatement } | { ForLoop: ForLoopStatement } | { Switch: SwitchStatement } | { WhileLoop: WhileStatement } | { DoWhileLoop: DoWhileStatement } | { TryCatch: TryCatchStatement } | { Return: ReturnStatement } | { Continue: [string | null, Span] } | { Break: [string | null, Span] } | { Throw: ThrowStatement } | { Comment: [string, Span] } | { MultiLineComment: [string, Span] } | { Labelled: { position: Span; name: string; statement: Statement } } | { VarVariable: VarVariableStatement } | { Empty: Span };

type ReturnStatement = [MultipleExpression | null, Span];

type ThrowStatement = [MultipleExpression, Span];

interface VarVariableStatement {
    declarations: VariableDeclarationItem<Expression | null>[];
    position: Span;
}

interface EnumDeclaration {
    is_constant: boolean;
    name: string;
    members: EnumMember[];
    position: Span;
}

type EnumMember = { Variant: { name: string; value: Expression | null; position: Span } };

interface InterfaceDeclaration {
    is_declare: boolean;
    name: StatementPosition;
    is_nominal: boolean;
    type_parameters: TypeParameter[] | null;
    extends: TypeAnnotation[] | null;
    members: WithComment<Decorated<InterfaceMember>>[];
    position: Span;
}

type Optionality = "Default" | "Optional" | "Required";

type TypeRule = "In" | "InKeyOf";

type InterfaceMember = { Method: { header: MethodHeader; name: PropertyKey<PublicOrPrivate>; type_parameters: TypeParameter[] | null; parameters: TypeAnnotationFunctionParameters; return_type: TypeAnnotation | null; is_optional: boolean; position: Span } } | { Property: { name: PropertyKey<PublicOrPrivate>; type_annotation: TypeAnnotation; is_readonly: boolean; is_optional: boolean; position: Span } } | { Indexer: { name: string; indexer_type: TypeAnnotation; return_type: TypeAnnotation; is_readonly: boolean; position: Span } } | { Constructor: { parameters: TypeAnnotationFunctionParameters; type_parameters: TypeParameter[] | null; return_type: TypeAnnotation | null; is_readonly: boolean; position: Span } } | { Caller: { parameters: TypeAnnotationFunctionParameters; type_parameters: TypeParameter[] | null; return_type: TypeAnnotation | null; is_readonly: boolean; position: Span } } | { Rule: { parameter: string; rule: TypeRule; matching_type: TypeAnnotation; optionality: Optionality; is_readonly: boolean; output_type: TypeAnnotation; position: Span } } | { Comment: [string, boolean, Span] };


	interface FunctionBase {
		type_parameters?: TypeParameter[],
		return_type?: TypeAnnotation,
		position: Span
	}



	interface ExpressionFunction extends FunctionBase {
		header: FunctionHeader,
		body: Block,
		name: ExpressionPosition,
		parameters: FunctionParameters<null, null>
	}


type BinaryOperator = "Add" | "Subtract" | "Multiply" | "Divide" | "Modulo" | "Exponent" | "BitwiseShiftLeft" | "BitwiseShiftRight" | "BitwiseShiftRightUnsigned" | "BitwiseAnd" | "BitwiseXOr" | "BitwiseOr" | "StrictEqual" | "StrictNotEqual" | "Equal" | "NotEqual" | "GreaterThan" | "LessThan" | "LessThanEqual" | "GreaterThanEqual" | "LogicalAnd" | "LogicalOr" | "NullCoalescing" | "Divides" | "Pipe" | "Compose";

type BinaryAssignmentOperator = "LogicalNullishAssignment" | "AddAssign" | "SubtractAssign" | "MultiplyAssign" | "DivideAssign" | "ModuloAssign" | "ExponentAssign" | "LogicalAndAssign" | "LogicalOrAssign" | "BitwiseShiftLeftAssign" | "BitwiseShiftRightAssign" | "BitwiseShiftRightUnsigned" | "BitwiseAndAssign" | "BitwiseXOrAssign" | "BitwiseOrAssign";

type UnaryOperator = "Plus" | "Negation" | "BitwiseNot" | "LogicalNot" | "Await" | "TypeOf" | "Void" | "Delete" | "Yield" | "DelegatedYield";

type IncrementOrDecrement = "Increment" | "Decrement";

type UnaryPrefixAssignmentOperator = "Invert" | { IncrementOrDecrement: IncrementOrDecrement };

type UnaryPostfixAssignmentOperator = IncrementOrDecrement;

interface TemplateLiteral {
    tag: Expression | null;
    parts: TemplateLiteralPart<Expression>[];
    position: Span;
}

type TemplateLiteralPart<T> = { Static: string } | { Dynamic: T };

type FunctionLocationModifier = "Server" | "Worker";

type FunctionHeader = { VirginFunctionHeader: { is_async: boolean; location: FunctionLocationModifier | null; generator_star_token_position: Span | null; position: Span } } | { ChadFunctionHeader: { is_async: boolean; is_generator: boolean; location: FunctionLocationModifier | null; position: Span } };

type MethodHeader = "Get" | "Set" | { Regular: { is_async: boolean; generator: GeneratorSpecifier | null } };

type GeneratorSpecifier = { Star: Span } | "Keyword";

type FunctionBody = Block | null;

type Visibility = "Private" | "Public" | "Protected";

type SourceId = number;

type SpanWithSource = BaseSpan<SourceId>;

type Span = BaseSpan<null>;

interface BaseSpan<T> {
    start: number;
    end: number;
    source: T;
}

/**
*/
declare class CheckOptions {
  free(): void;
/**
*/
  lsp_mode: boolean;
}



interface WASMCheckOutput {
	readonly diagnostics: DiagnosticsContainer
}
/**
*/
declare class WASMCheckOutput {
  free(): void;
/**
* @param {string} path
* @param {number} pos
* @returns {string}
*/
  get_type_at_position(path: string, pos: number): string;
}

type InitInput = RequestInfo | URL | Response | BufferSource | WebAssembly.Module;

interface InitOutput {
  readonly memory: WebAssembly.Memory;
  readonly __wbg_checkoptions_free: (a: number) => void;
  readonly __wbg_get_checkoptions_lsp_mode: (a: number) => number;
  readonly __wbg_set_checkoptions_lsp_mode: (a: number, b: number) => void;
  readonly experimental_build: (a: number, b: number, c: number, d: number) => number;
  readonly __wbg_wasmcheckoutput_free: (a: number) => void;
  readonly wasmcheckoutput_diagnostics: (a: number) => number;
  readonly wasmcheckoutput_get_type_at_position: (a: number, b: number, c: number, d: number, e: number) => void;
  readonly check: (a: number, b: number, c: number) => number;
  readonly check_with_options: (a: number, b: number, c: number, d: number) => number;
  readonly run_cli: (a: number, b: number, c: number, d: number, e: number) => void;
  readonly parse_expression: (a: number, b: number) => number;
  readonly parse_module: (a: number, b: number) => number;
  readonly parse_module_and_into_string: (a: number, b: number, c: number, d: number) => number;
  readonly just_imports: (a: number, b: number) => number;
  readonly get_version: () => number;
  readonly __wbindgen_malloc: (a: number, b: number) => number;
  readonly __wbindgen_realloc: (a: number, b: number, c: number, d: number) => number;
  readonly __wbindgen_add_to_stack_pointer: (a: number) => number;
  readonly __wbindgen_free: (a: number, b: number, c: number) => void;
  readonly __wbindgen_exn_store: (a: number) => void;
}

type SyncInitInput = BufferSource | WebAssembly.Module;
/**
* Instantiates the given `module`, which can either be bytes or
* a precompiled `WebAssembly.Module`.
*
* @param {SyncInitInput} module
*
* @returns {InitOutput}
*/
declare function initSync(module: SyncInitInput): InitOutput;

/**
* If `module_or_path` is {RequestInfo} or {URL}, makes a request and
* for everything else, calls `WebAssembly.instantiate` directly.
*
* @param {InitInput | Promise<InitInput>} module_or_path
*
* @returns {Promise<InitOutput>}
*/
declare function __wbg_init (module_or_path?: InitInput | Promise<InitInput>): Promise<InitOutput>;

export { type TypeAnnotationFunctionParameters as $, type ArrayElement as A, type BuildOutput as B, type Comments as C, type DiagnosticsContainer as D, type Expression as E, type FailedBuildOutput as F, type TypeAlias as G, type TypeParameter as H, type InExpressionLHS as I, type VariableDeclarationItem as J, type VariableDeclaration as K, type LHSOfAssignment as L, type MultipleExpression as M, type NumberRepresentation as N, type Output as O, type PropertyReference as P, type Quoted as Q, type VariableDeclarationKeyword as R, type SpecialOperators as S, type TypeCheckOptions as T, type TypeAnnotation as U, type VariableKeyword as V, type AnnotationWithBinder as W, type TupleElementKind as X, type TypeCondition as Y, type CommonTypes as Z, type TypeConditionResult as _, check_with_options as a, type VarVariableStatement as a$, type TypeAnnotationFunctionParameter as a0, type TypeAnnotationSpreadFunctionParameter as a1, type Decorator as a2, type Decorated as a3, type SpreadParameterName as a4, type ImportedItems as a5, type ImportDeclaration as a6, type ImportExportName as a7, type ImportPart as a8, type IsExpression as a9, type JSXAttribute as aA, type TryCatchStatement as aB, type DeclareVariableDeclaration as aC, type StatementFunction as aD, type StatementOrDeclaration as aE, type Block as aF, type BlockOrSingleStatement as aG, type Declaration as aH, type ImportLocation as aI, type Module as aJ, type SwitchStatement as aK, type SwitchBranch as aL, type WhileStatement as aM, type DoWhileStatement as aN, type Namespace as aO, type WithComment as aP, type VariableIdentifier as aQ, type VariableField as aR, type ArrayDestructuringField as aS, type ObjectDestructuringField as aT, type ClassDeclaration as aU, type IfStatement as aV, type ConditionalElseStatement as aW, type UnconditionalElseStatement as aX, type Statement as aY, type ReturnStatement as aZ, type ThrowStatement as a_, type Parameter as aa, type ParameterData as ab, type SpreadParameter as ac, type FunctionParameters as ad, type ThisParameter as ae, type SuperParameter as af, type AlwaysPublic as ag, type PublicOrPrivate as ah, type PropertyKey as ai, type ClassConstructor as aj, type ClassFunction as ak, type ObjectLiteralMethod as al, type ExceptionVarField as am, type IsStatic as an, type ClassMember as ao, type ClassProperty as ap, type ExportDeclaration as aq, type Exportable as ar, type ExportPart as as, type ObjectLiteral as at, type ObjectLiteralMember as au, type JSXRoot as av, type JSXElement as aw, type JSXElementChildren as ax, type JSXFragment as ay, type JSXNode as az, parse_module as b, type EnumDeclaration as b0, type EnumMember as b1, type InterfaceDeclaration as b2, type Optionality as b3, type TypeRule as b4, type InterfaceMember as b5, type FunctionBase as b6, type ExpressionFunction as b7, type BinaryOperator as b8, type BinaryAssignmentOperator as b9, type UnaryOperator as ba, type IncrementOrDecrement as bb, type UnaryPrefixAssignmentOperator as bc, type UnaryPostfixAssignmentOperator as bd, type TemplateLiteral as be, type TemplateLiteralPart as bf, type FunctionLocationModifier as bg, type FunctionHeader as bh, type MethodHeader as bi, type GeneratorSpecifier as bj, type FunctionBody as bk, type Visibility as bl, type SourceId as bm, type SpanWithSource as bn, type Span as bo, type BaseSpan as bp, CheckOptions as bq, WASMCheckOutput as br, type InitInput as bs, type InitOutput as bt, type SyncInitInput as bu, initSync as bv, __wbg_init as bw, check as c, parse_module_and_into_string as d, experimental_build as e, type Diagnostic as f, get_version as g, type DiagnosticKind as h, type TypeOrConst as i, just_imports as j, type FunctionArgument as k, type SuperReference as l, type ExpressionPosition as m, type StatementPosition as n, type NumberSign as o, parse_expression as p, type ToStringOptions as q, run_cli as r, type ParseOptions as s, type ArrowFunction as t, type IsAsync as u, type ExpressionOrBlock as v, type VariableOrPropertyAccess as w, type ForLoopStatement as x, type ForLoopStatementInitializer as y, type ForLoopCondition as z };
